<?php
namespace Teknisa\PAR\Factory;

use Doctrine\ORM\EntityManager;

class ParceiroNegocioFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function replicaInteprosp($paramsInsertInterlocutor){
        
        $insertInterlocutor = "INSERT INTO INTEPROSP (CDPROSPECT, CDINTEPROS, IDTPINTEPROS, NMINTEPROS, DSEMAILIPROS, NRTEL1INPROS, NRORG, DSCARACINPROS, IDINPROSRECPRES)
                               VALUES(:CDPROSPECT,
                                      :CDINTEPROS,
                                      'O',
                                      :NMINTEPROS,
                                      :DSEMAILIPROS,
                                      :NRTEL1INPROS,
                                      :NRORG,
                                      :DSCARACINPROS,
                                      'S')";
        
        try{
            $std = $this->connection->prepare($insertInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function updateInteprosp($paramsInsertInterlocutor){

        $updatetInterlocutor = "UPDATE INTEPROSP SET 
                            NMINTEPROS = :NMINTEPROS, DSEMAILIPROS = :DSEMAILIPROS, NRTEL1INPROS = :NRTEL1INPROS, DSCARACINPROS = :DSCARACINPROS WHERE CDPROSPECT = :CDPROSPECT AND CDINTEPROS = :CDINTEPROS AND NRORG = :NRORG";
        
        try{
            $std = $this->connection->prepare($updatetInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getCdintepros($nrrelacionaparc, $nrorg){
        
        $selectInterlocutor = " SELECT CDINTEPROS FROM INTEPROSP WHERE NRRELACIONAPARC = '$nrrelacionaparc' and NRORG = '$nrorg' ";
        
        try{
            $std = $this->connection->prepare($selectInterlocutor);
            $std->execute();
            $cdIntepros = $std->fetchAll();
            return $cdIntepros[0]['CDINTEPROS'];
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getNewParcInteprosp($params){
        
        $select = "SELECT PN.NRPARCNEGOCIO, RP.NRRELACIONAPARC FROM
                    PARCNEGOCIO PN
                    LEFT JOIN RELACIONAPARC RP
                    ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                    AND RP.NRORG = PN.NRORG
                    LEFT JOIN INTEPROSP IP
                    ON IP.NRRELACIONAPARC = RP.NRRELACIONAPARC
                    AND IP.NRORG = RP.NRORG
                    WHERE IP.CDPROSPECT = :CDPROSPECT
                    AND IP.CDINTEPROS = :CDINTEPROS
                    AND IP.NRORG = :NRORG";
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $newparc = $std->fetchAll();
            return $newparc[0];
        }
        catch(\Exception $e){
            throw $e;
        };
    }
    
    public function deleteInteprosp($nrrelacionaparc, $nrorg){
        
        $deleteInterlocutor = "DELETE FROM INTEPROSP WHERE NRRELACIONAPARC = '$nrrelacionaparc' and NRORG = '$nrorg' ";
        
        try{
            $std = $this->connection->prepare($deleteInterlocutor);
            $std->execute();
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function atualizarCaminhoLogo(array $params) {
        $sql = "UPDATE PARCNEGOCIO
                   SET DSDIRLOGOMARCA = :DSDIRLOGOMARCA
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    
       /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual(array $params){
        $sql = "SELECT NVL(SUM(NRPERCENTINDICE), 0) AS TOTPERCENT 
                  FROM FCT_INDICESPADRPARC
                 WHERE NRPARCNEGOCIO     = :NRPARCNEGOCIO
                   AND NRORG             = :NRORG
                   AND NRINDICESPADRPARC <> (NVL(:NRINDICESPADRPARC, -1))";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Valida se existe o registro de parametro de faturamento para o parceiro informado */
    public function validaRegistoParamFat(array $params){
        $sql = "SELECT COUNT(*) AS VALIDAPARFAT 
                  FROM PAR_PARAMFATURA
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG = :NRORG";
                 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $retorno = $std->fetchAll();
        return $retorno[0]["VALIDAPARFAT"];
    }
    
    /* Edita o registro da parametrização de faturamento do Parceiro de Negócio */
    public function updateParParamFatura(array $params){
        $sql = "UPDATE PAR_PARAMFATURA
                   SET CDFILIALFATURA = :CDFILIALFATURA, 
                       CDSERIENF = :CDSERIENF,
                       CDEMPRESA = :CDEMPRESA,
                       CDINSCESTA = :CDINSCESTA,
                       CDINSCMUNI = :CDINSCMUNI,
                       DTULTATU = SYSDATE,
                       NRORGULTATU = :NRORG
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG = :NRORG";
                 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* Inclui o registro da parametrização de faturamento do Parceiro de Negócio */
    public function insertParParamFatura(array $params){
        $sql = "INSERT INTO PAR_PARAMFATURA
                    (NRPARAMFATURA, NRPARCNEGOCIO, CDFILIALFATURA, CDSERIENF, CDEMPRESA, CDINSCESTA, CDINSCMUNI, 
                     IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO)
                VALUES
                    (:NRPARAMFATURA, :NRPARCNEGOCIO, :CDFILIALFATURA, :CDSERIENF, :CDEMPRESA, :CDINSCESTA, :CDINSCMUNI,
            		 'S', :NRORG, SYSDATE, :NRORG)";
                 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /* recebe os dados de parâmetro de Faturamento */
    public function recebeParamFatura(array $params){
        $sql = "SELECT PAR_PARAMFATURA.NRPARAMFATURA, PAR_PARAMFATURA.NRPARCNEGOCIO, PAR_PARAMFATURA.CDFILIALFATURA, FILIAL.NMFILIAL AS NMFILIALFATURA, PAR_PARAMFATURA.CDSERIENF, SERIENF.DSSERIE,
                       PAR_PARAMFATURA.CDEMPRESA, PAR_PARAMFATURA.CDINSCESTA, PAR_PARAMFATURA.CDINSCMUNI, PAR_PARAMFATURA.IDATIVO, PAR_PARAMFATURA.NRORG, PAR_PARAMFATURA.DTINCLUSAO, 
                       PAR_PARAMFATURA.CDOPERINCLUSAO, PAR_PARAMFATURA.NRORGINCLUSAO, PAR_PARAMFATURA.DTULTATU, PAR_PARAMFATURA.CDOPERULTATU, PAR_PARAMFATURA.NRORGULTATU
                  FROM PAR_PARAMFATURA
                  LEFT JOIN FILIAL 
                    ON FILIAL.CDFILIAL = PAR_PARAMFATURA.CDFILIALFATURA
                   AND FILIAL.NRORG = PAR_PARAMFATURA.NRORG
                  LEFT JOIN SERIENF
                    ON SERIENF.CDSERIE = PAR_PARAMFATURA.CDSERIENF
                   AND SERIENF.CDEMPRESA = PAR_PARAMFATURA.CDEMPRESA
                   AND SERIENF.CDINSCESTA = PAR_PARAMFATURA.CDINSCESTA
                   AND SERIENF.CDINSCMUNI = PAR_PARAMFATURA.CDINSCMUNI
                   AND SERIENF.NRORG = PAR_PARAMFATURA.NRORG
                 WHERE PAR_PARAMFATURA.NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND PAR_PARAMFATURA.NRORG = :NRORG";
                 
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}