<?php
namespace Teknisa\PAR\Service;

use Teknisa\Model\Pedidosexpedicao\Entities\Transport;
use Teknisa\Model\Crm\Entities\Prospect;
use Teknisa\Model\Geral\Entities\Cliente;
use Teknisa\Model\Geral\Entities\Enderecoparc;
use Teknisa\Model\Geral\Entities\Fornecedor;
use Teknisa\Model\Geral\Entities\Parceria;
use Teknisa\Model\Geral\Entities\Parcnegocio;
use Teknisa\Model\Geral\Entities\Relacionaparc;
use Teknisa\Model\Geral\Entities\Comunicaparc;
use Teknisa\Model\Geral\Entities\Formacomu;
use Teknisa\Model\Geral\Entities\Vendedor;
use Teknisa\Model\Suprimentos\Entities\Paramforne;
use Teknisa\PAR\Service\EnderecoParceiroService;
use Teknisa\PAR\Factory\ParceiroNegocioFactory;

class ParceiroNegocioService {

    protected $enderecoService;
    protected $leadService;
    protected $parcNegocioFactory;
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $defaultTipos = [
        "CLIENTE",
        "PROSPECT",
        "PESSOA",
        "OPERADOR",
        "FORNECEDOR",
        "TRANSPORTADORA",
        "VENDEDOR",
        "INTERLOCUTOR",
        "REVENDA"
    ];

    public function __construct($entityManager, $environment, $newCode, EnderecoParceiroService $enderecoService, $leadService, ParceiroNegocioFactory $parcNegocioFactory) {
        $this->entityManager      = $entityManager;
        $this->environment        = $environment;
        $this->newCode            = $newCode;
        $this->enderecoService    = $enderecoService;
        $this->leadService        = $leadService;
        $this->parcNegocioFactory = $parcNegocioFactory;
    }

    /*
     *
     *  Funções de manipulação de dados
     *
     */
    public function saveParceiro($row) {
        $this->validateParceiro($row);
        
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();

        $nrinscricaoparc = $row["NRINSCRICAOPARC"];
        $cdtipoinscricao = $row["CDTIPOINSCRICAO"];
        $parcnegocioRepository = $this->entityManager->getRepository(Parcnegocio::class);
        
        $this->entityManager->beginTransaction();
        try {
            if (!empty($row["NRPARCNEGOCIO"])) {
                $parcnegocio = $parcnegocioRepository->findOneBy(array(
                   "nrparcnegocio" => $row["NRPARCNEGOCIO"],
                   "nrorg" => $nrorg
                ));
            } else {
                $parcnegocio = $parcnegocioRepository->findOneBy(array(
                   "nrinscricaoparc" => $nrinscricaoparc,
                   "cdtipoinscricao" => $cdtipoinscricao,
                   "nrorg" => $nrorg
                ));
            }

            if ($parcnegocio === null) {
                $parcnegocio = new Parcnegocio();
                $parcnegocio->setNrorg($nrorg);
                $parcnegocio->setNrparcnegocio($this->newCode->getCodeByTableName('PARCNEGOCIO', $nrorg, 1, 10, true));
                $parcnegocio->setDtinclusao($currentTime);
                $parcnegocio->setCdoperinclusao($operador);
                $parcnegocio->setNrorginclusao($nrorg);
                $parcnegocio->setIdparcfundido('N');
                $nrparcnegocio = $row["NRPARCNEGOCIO"] = $parcnegocio->getNrparcnegocio();
            } else {
                $parcnegocio->setDtultatu($currentTime);
                $parcnegocio->setCdoperultatu($operador);
                $parcnegocio->setNrorgultatu($nrorg);
            }
            
            $parcnegocio->setNrinscricaoparc($nrinscricaoparc);
            $parcnegocio->setCdtipoinscricao($cdtipoinscricao);
            $parcnegocio->setIdpessoafisica($row["IDPESSOAFISICA"]);
            $parcnegocio->setIdinstituicao($row["IDINSTITUICAO"]);
            $parcnegocio->setNmprincipalparc($row["NMPRINCIPALPARC"]);
            $parcnegocio->setNmsecundariparc($row["NMSECUNDARIPARC"]);
            $parcnegocio->setCdtipoparcprincipal($row["CDTIPOPARCPRINCIPAL"]);
            $parcnegocio->setNrseqtpativcli(!empty($row["NRSEQTPATIVCLI"]) ? $row["NRSEQTPATIVCLI"] : null);
            $parcnegocio->setCdsegmer(!empty($row["CDSEGMER"]) ? $row["CDSEGMER"] : null);
            $parcnegocio->setCdativcli(!empty($row["CDATIVCLI"]) ? $row["CDATIVCLI"] : null);
            $parcnegocio->setNrtpclassificaparc(isset($row["NRTPCLASSIFICAPARC"]) ? $row["NRTPCLASSIFICAPARC"] : null);
            $parcnegocio->setIdativo($row["IDATIVO"]);
            $parcnegocio->setCdoperresponsavel(!empty($row["CDOPERRESPONSAVEL"]) ? $row["CDOPERRESPONSAVEL"] : null);
            $dtnascifundparc = null;
            
            if (!empty($row["DTNASCIFUNDPARC"])) {
                $format = strpos($row["DTNASCIFUNDPARC"], ":") === false ? "d/m/Y" : "d/m/Y H:i:s";
                $dtnascifundparc = \DateTime::createFromFormat($format, $row["DTNASCIFUNDPARC"]);
            }
            
            $parcnegocio->setDtnascifundparc($dtnascifundparc);
            $this->entityManager->persist($parcnegocio);
            $this->entityManager->flush();
            $this->replicaParceiro($row);
            $this->entityManager->commit();
        } catch(\Exception $e) {
            if ($row['__is_new']) 
                $this->rollbackNovaParceria($nrparcnegocio, $nrorg);
            else
                $this->entityManager->rollback();
            throw $e;
        }
        
        return $parcnegocio->getNrparcnegocio();
    }

    private function replicaParceiro($row) {
        $nrorg       = $this->environment->getNrorgTrab();
        $operador    = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();

        $parcnegocioRepository   = $this->entityManager->getRepository(Parcnegocio::class);
        $parceriaRepository      = $this->entityManager->getRepository(Parceria::class);
        $clienteRepository       = $this->entityManager->getRepository(Cliente::class);
        $fornecedorRepository    = $this->entityManager->getRepository(Fornecedor::class);
        $relacionaparcRepository = $this->entityManager->getRepository(Relacionaparc::class);
        $vendedorRepository      = $this->entityManager->getRepository(Vendedor::class);
        $prospectRepository      = $this->entityManager->getRepository(Prospect::class);
        $transportRepository     = $this->entityManager->getRepository(Transport::class);
        $enderecoparcRepository  = $this->entityManager->getRepository(Enderecoparc::class);

        $tipoinscricao = array(
            'CPF'    => 'F',
            'CNPJ'   => 'J',
            'LIVRE'  => 'I',
            'CODIGO' => 'O'
        );
        
        $tiposParceiro   = $row["TIPODEPARCEIRO"];
        $cdtipoinscricao = $row["CDTIPOINSCRICAO"];
        $nrinscricaoparc = $row["NRINSCRICAOPARC"];
        $nmprincipalparc = $row["NMPRINCIPALPARC"];
        $nmsecundariparc = $row["NMSECUNDARIPARC"];
        $nrparcnegocio   = $row["NRPARCNEGOCIO"];
        if (!$nrparcnegocio)
            throw new \Exception("Ocorreu um erro ao buscar número do parceiro de negócio.");

        $dtnascifundparc = null;
        if (!empty($row["DTNASCIFUNDPARC"])) {
            $format = strpos($row["DTNASCIFUNDPARC"], ":") === false ? "d/m/Y" : "d/m/Y H:i:s";
            $dtnascifundparc = \DateTime::createFromFormat($format, $row["DTNASCIFUNDPARC"]);
        }
        
        $cdsegmer = !empty($row["CDSEGMER"]) ? $row["CDSEGMER"] : null;
        foreach ($tiposParceiro as $tipo) {
            $parceria = $parceriaRepository->findOneBy(array(
                "nrparcnegocio"  => $nrparcnegocio,
                "cdtipoparceiro" => $tipo,
                "nrorg"          => $nrorg
            ));
            
            if ($parceria === null) {
                $parceria = new Parceria();
                $parceria->setNrparceria($this->newCode->getCodeByTableName('PARCERIA', $nrorg, 1, 10, true));
                $parceria->setNrorg($nrorg);
                $parceria->setNrparcnegocio($nrparcnegocio);
                $parceria->setCdtipoparceiro($tipo);
                $parceria->setDtinclusao($currentTime);
                $parceria->setCdoperinclusao($operador);
                $parceria->setNrorginclusao($nrorg);
                $parceria->setIdativo($row["IDATIVO"]);
            } else {
                $parceria->setDtultatu($currentTime);
                $parceria->setCdoperultatu($operador);
                $parceria->setNrorgultatu($nrorg);
                $parceria->setIdativo($row["IDATIVO"]);
                
                if ($this->isNewItem($parceria)) {
                    $parceria->setNrparceria($this->newCode->getCodeByTableName('PARCERIA', $nrorg, 1, 10, true));
                    $parceria->setNrparcnegocio($nrparcnegocio);
                    $parceria->setCdtipoparceiro($tipo);
                }
            }
            
            $this->entityManager->persist($parceria);
        }
        
        $this->entityManager->flush();
        // Seta o IDATIVO como N os tipos de parceria não selecionados, mas já existentes
        $tiposToDelete = array_diff($this->defaultTipos, $tiposParceiro);
        foreach ($tiposToDelete as $tipo) {
            $parceria = $parceriaRepository->findOneBy(array(
                "nrparcnegocio"  => $nrparcnegocio,
                "cdtipoparceiro" => $tipo,
                "nrorg"          => $nrorg
            ));
            
            if ($parceria !== null) {
                $parceria->setDtultatu($currentTime);
                $parceria->setCdoperultatu($operador);
                $parceria->setNrorgultatu($nrorg);
                $parceria->setIdativo('N');
                $this->entityManager->persist($parceria);
                $this->entityManager->flush();
            }
        }

        if (in_array("CLIENTE", $tiposParceiro)) {
            $cliente = $clienteRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($cliente === null) {
                $cliente = new Cliente();
                $cliente->setNrparcnegocio($nrparcnegocio);
                $clienteAnterior = $clienteRepository->findOneBy(array(
                    "nrinsjurclie" => $nrinscricaoparc,
                    "nrorg"        => $nrorg
                ));
                
                if ($clienteAnterior === null) 
                    $cliente->setCdcliente($tipoinscricao[$cdtipoinscricao].$nrinscricaoparc);
                else 
                    $cliente->setCdcliente($this->newCode->getCodeByTableName('CLIENTE', $nrorg));
                
                $cliente->setIdtpijurclie($tipoinscricao[$cdtipoinscricao]);
                $cliente->setNrinsjurclie($nrinscricaoparc);
                $cliente->setNmrazsocclie($nmprincipalparc);
                $cliente->setNmfantclie($nmsecundariparc);
                $cliente->setCdsegmer($cdsegmer);
                $cliente->setDtnascclie($dtnascifundparc);
                $cliente->setNrorg($nrorg);
                $cliente->setIdpripubclie('V');
                $cliente->setIdversaldcli('N');
                $cliente->setIdutsaldcli('N');
                $cliente->setIdcalcdvclie('1');
                $cliente->setIdutsubclien('N');
                $cliente->setIdzersaldcon('S');
                $cliente->setIdgernfclie('1');
                $cliente->setIdsitfinclie('N');
                $cliente->setIdligextclie('S');
                $cliente->setIdtplimatefat('P');
                $cliente->setNrlimatemes(10);
                $cliente->setIdlibatdcliinad('N');
                $cliente->setIdpgfrettrancli('9');
                $cliente->setIdcomparqxmlcli('S');
                $cliente->setIdclisecatclpri('N');
                $cliente->setIdgravobscliped('S');
                $cliente->setIdexbtpnomeclie('F');
                $cliente->setIdutdescicms('N');
                $cliente->setIdativo($row["IDATIVO"]);
                $cliente->setIdclieativ($row["IDATIVO"]);
                $cliente->setDtinclusao($currentTime);
                $cliente->setCdoperinclusao($operador);
                $cliente->setNrorginclusao($nrorg);
                $prospect = $prospectRepository->findOneBy(array(
                    "nrparcnegocio" => $nrparcnegocio,
                    "nrorg"         => $nrorg
                ));
                
                if($prospect !== null){
                    $prospect->setCdcliente($cliente->getCdcliente());
                    $this->entityManager->persist($prospect);
                }
                
                $this->entityManager->persist($cliente);
            } else {
                $cliente->setIdativo($row["IDATIVO"]);
                $cliente->setIdclieativ($row["IDATIVO"]);
                $this->entityManager->persist($cliente);
            }
            
            $this->entityManager->flush();
        } else {
            $cliente = $clienteRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if($cliente !== null){
                $cliente->setIdativo('N');
                $cliente->setIdclieativ('N');
                $this->entityManager->persist($cliente);
                $this->entityManager->flush();
            }
        }

        if (in_array("FORNECEDOR", $tiposParceiro)) {
            $fornecedor = $fornecedorRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg" => $nrorg
            ));
            if ($fornecedor === null) {
                $fornecedor    = $fornecedorRepository->findOneBy(array(
                    "nrinsjurforn" => $nrinscricaoparc,
                    "nrorg"        => $nrorg
                ));
                
                if ($fornecedor === null) {
                    $fornecedor = new Fornecedor();
                    $fornecedor->setNrparcnegocio($nrparcnegocio);
                    $fornecedor->setCdforneced($tipoinscricao[$cdtipoinscricao].$nrinscricaoparc);
                    $fornecedor->setIdtpijurforn($tipoinscricao[$cdtipoinscricao]);
                    $fornecedor->setNrinsjurforn($nrinscricaoparc);
                    $fornecedor->setNmrazsocforn($nmprincipalparc);
                    $fornecedor->setNmfantforn($nmsecundariparc);
                    $fornecedor->setIdpripubforn('V');
                    $fornecedor->setIdagrubolfor('N');
                    $fornecedor->setIdligextforn('N');
                    $fornecedor->setIdutqtdispforn('N');
                    $fornecedor->setIdbloqcmpsaffor('N');
                    $fornecedor->setIdlocalforncad('F');
                    $fornecedor->setIdsexoprofi('I');
                    $fornecedor->setIdprofipossveic('N');
                    $fornecedor->setIdavaliaprofi('0');
                    $fornecedor->setIdnpermancotint('N');
                    $fornecedor->setIdcomparqxmlfor('S');
                    $fornecedor->setIdtipopagaprof('CONTAPAGAR');
                    $fornecedor->setNrorg($nrorg);
                    $fornecedor->setIdativo($row["IDATIVO"]);
                    $fornecedor->setIdsitucada('A');
                    $fornecedor->setDtinclusao($currentTime);
                    $fornecedor->setCdoperinclusao($operador);
                    $fornecedor->setNrorginclusao($nrorg);
                    $this->entityManager->persist($fornecedor);
                } else {
                    $fornecedor->setNrparcnegocio($nrparcnegocio);
                    $fornecedor->setIdativo($row["IDATIVO"]);
                    $fornecedor->setIdsitucada('A');
                    $this->entityManager->persist($fornecedor);
                }
            } else {
                $fornecedor->setIdativo($row["IDATIVO"]);
                $fornecedor->setIdsitucada('A');
                $this->entityManager->persist($fornecedor);
            }
            
            $this->entityManager->flush();
        } else {
            $fornecedor = $fornecedorRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($fornecedor !== null) {
                $fornecedor->setIdativo('N');
                $fornecedor->setIdsitucada('I');
                $this->entityManager->persist($fornecedor);
                $this->entityManager->flush();
            }
        }

        if (in_array("INTERLOCUTOR", $tiposParceiro)) {
            $interlocutor = $relacionaparcRepository->findOneBy(array(
                "nrparcnegrelac" => $nrparcnegocio,
                "nrorg"          => $nrorg
            ));
            
            if ($interlocutor !== null) {
                $interlocutor->setIdativo($row["IDATIVO"]);
                $this->entityManager->persist($interlocutor);
            }
            
            $this->entityManager->flush();
        } else {
            $interlocutor = $relacionaparcRepository->findOneBy(array(
                "nrparcnegrelac" => $nrparcnegocio,
                "nrorg"          => $nrorg
            ));
            if ($interlocutor !== null) {
                $interlocutor->setIdativo('N');
                $this->entityManager->persist($interlocutor);
                $this->entityManager->flush();
            }
        }

        if (in_array("VENDEDOR", $tiposParceiro)) {
            $vendedor = $vendedorRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($vendedor === null) {
                $vendedor = new Vendedor();
                $vendedor->setNrparcnegocio($nrparcnegocio);
                $vendedor->setNmrazsocven($nmprincipalparc);
                $vendedor->setNmfanven($nmsecundariparc);
                $vendedor->setNrorg($nrorg);
                $vendedor->setNrinsjurven($nrinscricaoparc);
                $vendedor->setCdvendedor($this->newCode->getCodeByTableName('CDVENDEDOR', $nrorg, 1, 4));
                $vendedor->setIdtpijurven($tipoinscricao[$cdtipoinscricao]);
                $vendedor->setIdligextvend('N');
                $vendedor->setIdgeracomisvend('S');
                $vendedor->setIdcargovend('V');
                $vendedor->setIdativo($row["IDATIVO"]);
                $vendedor->setDtinclusao($currentTime);
                $vendedor->setCdoperinclusao($operador);
                $vendedor->setNrorginclusao($nrorg);
                $this->entityManager->persist($vendedor);
            }
            else {
                $vendedor->setIdativo($row["IDATIVO"]);
                $this->entityManager->persist($vendedor);
            }
            
            $this->entityManager->flush();
        } else {
            $vendedor = $vendedorRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if($vendedor !== null){
                $vendedor->setIdativo('N');
                $this->entityManager->persist($vendedor);
                $this->entityManager->flush();
            }
        }
        
        if (in_array("PROSPECT", $tiposParceiro)) {
            $prospect = $prospectRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg" => $nrorg
            ));
            
            if ($prospect !== null) {
                $prospect->setNrorgultatu($nrorg);
                $prospect->setDtultatu($currentTime);
            } else {
                $prospect = new Prospect();
                $prospect->setCdprospect($this->newCode->getCodeByTableName('PROSPECT', $nrorg));
                $prospect->setNrorginclusao($nrorg);
                $prospect->setDtinclusao($currentTime);
                $prospect->setNrorg($nrorg);
                $prospect->setIdparcfundido('N');
            }

            $prospect->setNrparcnegocio($nrparcnegocio);
            $prospect->setNmrazsocpros($nmprincipalparc);
            $prospect->setNmfantpros($nmsecundariparc);
            $prospect->setIdtpijurpros($tipoinscricao[$cdtipoinscricao]);
            $prospect->setNrparcresprosp($row["NRPARCNEGOCIORESP"]);
            $prospect->setNrinsjurpros($row["NRINSCRICAOPARCFORMAT"]);
            $prospect->setIdativo($row["IDATIVO"]);
            $prospect->setIdligextpros("S");
            $prospect->setIdutlalvpros("N");
            $prospect->setIdpripubpros("V");
            $this->entityManager->persist($prospect);
            $this->entityManager->flush();
        } else {
            $prospect = $prospectRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($prospect !== null) {
                $prospect->setIdativo('N');
                $this->entityManager->persist($prospect);
                $this->entityManager->flush();
            }
        }
        
        if (in_array("TRANSPORTADORA", $tiposParceiro)) {
            $transport = $transportRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($transport === null) {
                $fornecedor = $fornecedorRepository->findOneBy(array(
                    "nrparcnegocio" => $nrparcnegocio,
                    "nrorg"         => $nrorg
                ));
                
                $cdforneced = $fornecedor->getCdforneced();
                $transport = $transportRepository->findOneBy(array(
                    "cdforneced"   => $cdforneced,
                    "idtpijurtran" => $tipoinscricao[$cdtipoinscricao],
                    "nrorg"        => $nrorg
                ));
                
                if($transport === null) {
                    $transport = new Transport();
                    $transport->setCdforneced($cdforneced);
                    $transport->setNrorg($nrorg);
                    $transport->setIdcomparqxmltra('S');
                    $transport->setNrparcnegocio($nrparcnegocio);
                    $transport->setDtinclusao($currentTime);
                    $transport->setCdoperinclusao($operador);
                    $transport->setNrorginclusao($nrorg);
                } else {
                    $transport->setNrparcnegocio($nrparcnegocio);
                    $transport->setDtultatu($currentTime);
                    $transport->setCdoperultatu($operador);
                    $transport->setNrorgultatu($nrorg);
                }
            } else {
                $transport->setDtultatu($currentTime);
                $transport->setCdoperultatu($operador);
                $transport->setNrorgultatu($nrorg);
            }
            
            $transport->setIdtpijurtran($tipoinscricao[$cdtipoinscricao]);
            $transport->setNrinsjurtran($nrinscricaoparc);
            $transport->setNmrazsoctran($nmprincipalparc);
            $transport->setNmfanttran($nmsecundariparc);
            $transport->setNrinsesttran($row['NRINSESTTRAN']);
            $transport->setDsendetran($row['ENDERECOTRAN']);
            $transport->setNmbairtran($row['NMBAIRROTRAN']);
            $transport->setNrceptran($row['NRCEPTRAN']);
            $transport->setCdpais($row['CDPAIS']);
            $transport->setSgestado($row['SGESTADO']);
            $transport->setCdmunicipio($row['CDMUNICIP']);
            $transport->setIdativo($row["IDATIVO"]);
            $this->entityManager->persist($transport);
            $this->entityManager->flush();
        } else {
            $transport = $transportRepository->findOneBy(array(
                "nrparcnegocio" => $nrparcnegocio,
                "nrorg"         => $nrorg
            ));
            
            if ($transport !== null){
                $transport->setIdativo('N');
                 $this->entityManager->persist($transport);
                 $this->entityManager->flush();
            }
        }
        
        $enderecosArray = $enderecoparcRepository->findBy(array(
            "nrparcnegocio" => $nrparcnegocio,
            "nrorg"         => $nrorg
        ));
        
        if ($enderecosArray != null) {
            foreach ($enderecosArray as $enderecoParc) {
                $this->enderecoService->replicaEnderecoParceiro($enderecoParc, $row);
            }
        } else {
            if(isset($row["ENDERECO"])){
                $enderecoParc = $row["ENDERECO"];
                $enderecoParc["NRPARCNEGOCIO"] = $nrparcnegocio;
                //$this->enderecoService->saveEnderecoParceiro($enderecoParc);
            }
        }
        
        $this->entityManager->flush();
    }

    private function rollbackNovaParceria($nrparcnegocio, $nrorg) {
        $params = array(
            "nrparcnegocio" => $nrparcnegocio,
            "nrorg" => $nrorg
        );
        $parcnegocioRepository = $this->entityManager->getRepository(Parcnegocio::class);
        $parcnegocio = $parcnegocioRepository->findOneBy($params);
        if ($nrparcnegocio !== null) {

            $parceriaRepository = $this->entityManager->getRepository(Parceria::class);
            $clienteRepository = $this->entityManager->getRepository(Cliente::class);
            $fornecedorRepository = $this->entityManager->getRepository(Fornecedor::class);
            $paramForneRepository = $this->entityManager->getRepository(Paramforne::class);
            $relacionaparcRepository = $this->entityManager->getRepository(Relacionaparc::class);
            $vendedorRepository = $this->entityManager->getRepository(Vendedor::class);
            $prospectRepository = $this->entityManager->getRepository(Prospect::class);
            $transportRepository = $this->entityManager->getRepository(Transport::class);
            $enderecoparcRepository = $this->entityManager->getRepository(Enderecoparc::class);

            $enderecosArray = $enderecoparcRepository->findBy($params);
            if ($enderecosArray != null) {
                foreach ($enderecosArray as $enderecoParc) {
                    $this->enderecoService->removeEnderecoParceiro(array("NRENDERECOPARC" => $enderecoParc->getNrenderecoparc()));
                }
            }

            $parcerias = $parceriaRepository->findBy($params);
            if ($parcerias != null) {
                foreach ($parcerias as $parceria) {
                    if ($parceria->getCdtipoparceiro() === "TRANSPORTADORA") {
                        $auxEntity = $transportRepository->findOneBy($params);
                        if ($auxEntity !== null) {
                            $this->entityManager->remove($auxEntity);
                            $this->entityManager->flush();
                        }
                    }
                }
                foreach ($parcerias as $parceria) {
                    $tipoParceiro = $parceria->getCdtipoparceiro();
                    switch ($tipoParceiro) {
                        case "CLIENTE":
                            $auxEntity = $clienteRepository->findOneBy($params);
                            break;
                        case "PROSPECT":
                            $auxEntity = $prospectRepository->findOneBy($params);
                            break;
                        case "FORNECEDOR":
                            $auxEntity = $fornecedorRepository->findOneBy($params);
                            break;
                        case "TRANSPORTADORA":
                            $auxEntity = $transportRepository->findOneBy($params);
                            break;
                        case "VENDEDOR":
                            $auxEntity = $vendedorRepository->findOneBy($params);
                            break;
                        case "INTERLOCUTOR":
                            $auxEntity = $relacionaparcRepository->findOneBy($params);
                            break;
                        default:
                            $auxEntity = null;
                    }
                    if ($auxEntity !== null) {
                        if ($tipoParceiro === "FORNECEDOR") {
                            $paramForneEntity = $paramForneRepository->findOneBy(array(
                                "cdforneced" => $auxEntity->getCdforneced(),
                                "nrorg" => $nrorg
                            ));
                            if ($paramForneEntity !== null) {
                                $this->entityManager->remove($paramForneEntity);
                                $this->entityManager->flush();
                            }
                        }
                        $this->entityManager->remove($auxEntity);
                    }
                    $this->entityManager->flush();
                    $this->entityManager->remove($parceria);
                }
            }
            
            $this->entityManager->remove($parcnegocio);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function saveContato($row) {
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();
        $tipoParceiro = $row['TIPODEPARCEIRO'];
        $parcnegocioPai             = $row['NRPARCNEGOCIO'];
        $row['NRPARCNEGOCIO']       = $this->newCode->getCodeByTableName('PARCNEGOCIO', $nrorg, 1, 10, true);
        $row['NMPRINCIPALPARC']     = $row['NOMECONT'];
        $row['CDTIPOINSCRICAO']     = 'LIVRE';
        $row['NRINSCRICAOPARC']     = '1';
        $row['NMSECUNDARIPARC']     = $row['PARENTTYPE'].' - '.$row['NOMEPARC'];
        $row['IDINSTITUICAO']       = 'N';
        $row['IDPESSOAFISICA']      = 'S';
        $row['CDTIPOPARCPRINCIPAL'] = $tipoParceiro;
        $row['TIPODEPARCEIRO']      = array($tipoParceiro); 
        $row['NRTPCLASSIFICAPARC']  = '';
        $row['contato']             = true;
        $this->entityManager->beginTransaction();
        
        try {
            $formacomuRepository = $this->entityManager->getRepository(Formacomu::class);
            $relacionaparcRepository = $this->entityManager->getRepository(Relacionaparc::class);
            $prospectRepository = $this->entityManager->getRepository(Prospect::class);
            $comunicaparcRepository  = $this->entityManager->getRepository(Comunicaparc::class);
            $parcnegocioRepository   = $this->entityManager->getRepository(Parcnegocio::class);
            
            if($tipoParceiro === 'INTERLOCUTOR' && $row['PARENTTYPE'] === 'PROSPECT'){
                $prospect = $prospectRepository->findOneBy(array(
                    "nrparcnegocio" => $parcnegocioPai,
                    "nrorg" => $nrorg
                ));
                
                $cdProspect = $prospect->getCdprospect();
                $relaciona = $relacionaparcRepository->findOneBy(array(
                       "nrparcnegocio"  => $parcnegocioPai,
                       "nrparcnegrelac" => $row['NRPARCNEGRELAC'],
                       "nrorg" => $nrorg
                ));
                
                if($relaciona === null){
                    $paramsNewCode = array(
                        "CDPROSPECT"   => $cdProspect,
                        "NRORG"        => $nrorg
                    );
                    $cdIntepros = $this->leadService->callForGetNewProspInterlocutorCode($paramsNewCode);
                    $paramsInsertInterlocutor = array(
                        "CDPROSPECT"    => $cdProspect,
                        "CDINTEPROS"    => $cdIntepros,
                        "NMINTEPROS"    => $row['NOMECONT'],
                        "DSEMAILIPROS"  => $row['EMAIL'] ? $row['EMAIL'] : '', 
                        "NRTEL1INPROS"  => $row['TELEFONE'] ? $row['TELEFONE'] : '',
                        "DSCARACINPROS" => $row['DSCARACINPROS'] ? $row['DSCARACINPROS'] : '',
                        "NRORG"         => $nrorg
                    );
                    $parcnegocioContato = $this->parcNegocioFactory->replicaInteprosp($paramsInsertInterlocutor);
                }else{
                    $cdintepros = $this->parcNegocioFactory->getCdintepros($relaciona->getNrrelacionaparc(), $nrorg);
                    $paramsUpdateInterlocutor = array(
                        "NMINTEPROS"   => $row['NOMECONT'],
                        "DSEMAILIPROS" => $row['EMAIL'] ? $row['EMAIL'] : '', 
                        "NRTEL1INPROS" => $row['TELEFONE'] ? $row['TELEFONE'] : '',
                        "CDPROSPECT"   => $cdProspect,
                        "CDINTEPROS"   => $cdintepros,
                        "DSCARACINPROS" => $row['DSCARACINPROS'] ? $row['DSCARACINPROS'] : '',
                        "NRORG"        => $nrorg
                    );
                    $parcnegocioContato = $this->parcNegocioFactory->updateInteprosp($paramsUpdateInterlocutor); 
                }
            }else{
                if(empty($row['NRPARCNEGRELAC'])){
                    $parcnegocioContato = $this->saveParceiro($row);
                }else{
                    $parcnegocioContato = $row['NRPARCNEGRELAC'];
                }
                if($parcnegocioContato){
                    if($row['EMAIL']){
                        $comunica = $comunicaparcRepository->findOneBy(array(
                           "nrcomunicaparc" => $row['NREMAIL'],
                           "nrorg" => $nrorg
                        ));
                        if($comunica === null){
                            $comunica = new Comunicaparc();
                            $comunica->setNrparcnegocio($parcnegocioContato);
                            $comunica->setNrcomunicaparc($this->newCode->getCodeByTableName('COMUNICAPARC', $nrorg, 1, 10, true));
                            $comunica->setNrorg($nrorg);
                            $comunica->setDtinclusao($currentTime);
                            $comunica->setCdoperinclusao($operador);
                            $comunica->setNrorginclusao($nrorg);
                            $comunica->setCdformacomu('05');
                        }else{
                            $comunica->setDtultatu($currentTime);
                            $comunica->setCdoperultatu($operador);
                            $comunica->setNrorgultatu($nrorg);
                        }
                        $comunica->setCdcomunicaparc($row['EMAIL']);
                        $comunica->setIdativo('S');
                        $this->entityManager->persist($comunica);
                        $this->entityManager->flush();
                    }
                    if($row['TELEFONE']){
                        $comunica = $comunicaparcRepository->findOneBy(array(
                           "nrcomunicaparc" => $row['NRTELEFONE'],
                           "nrorg" => $nrorg
                        ));
                        if($comunica === null){
                            $comunica = new Comunicaparc();
                            $comunica->setNrparcnegocio($parcnegocioContato);
                            $comunica->setNrorg($nrorg);
                            $comunica->setNrcomunicaparc($this->newCode->getCodeByTableName('COMUNICAPARC', $nrorg, 1, 6));
                            $comunica->setDtinclusao($currentTime);
                            $comunica->setCdoperinclusao($operador);
                            $comunica->setNrorginclusao($nrorg);
                            $comunica->setCdformacomu('01');
                        }else{
                            $comunica->setDtultatu($currentTime);
                            $comunica->setCdoperultatu($operador);
                            $comunica->setNrorgultatu($nrorg);
                        }
                        $comunica->setCdcomunicaparc($row['TELEFONE']);
                        $comunica->setIdativo('S');
                        $this->entityManager->persist($comunica);
                        $this->entityManager->flush();
                    }
                    $relparc = $relacionaparcRepository->findOneBy(array(
                       "nrrelacionaparc" => $row["NRRELACIONAPARC"],
                       "nrorg" => $nrorg
                    ));
                    if($relparc === null){
                        $relparc = new Relacionaparc();
                        $relparc->setNrparcnegocio($parcnegocioPai);
                        $relparc->setNrparcnegrelac($parcnegocioContato);
                        $relparc->setNrorg($nrorg);
                        $relparc->setNrrelacionaparc($this->newCode->getCodeByTableName('RELACPARCNEG', $nrorg, 1, 10, true));
                        $relparc->setDtinclusao($currentTime);
                        $relparc->setCdoperinclusao($operador);
                        $relparc->setNrorginclusao($nrorg);
                    }
                    else{
                        $relparc->setDtultatu($currentTime);
                        $relparc->setCdoperultatu($operador);
                        $relparc->setNrorgultatu($nrorg);    
                    }
                    $relparc->setNrtiporelaciona($row['NRTIPORELACIONA']);
                    $relparc->setIdativo('S');
                    //var_dump($relparc);
                    $this->entityManager->persist($relparc);    
                    $this->entityManager->flush();
                    
                    $parcnegocioContato = array(
                        'NRPARCNEGOCIO' => $parcnegocioContato,
                        'NRRELACIONAPARC' => $relparc->getNrrelacionaparc()
                    );
                }
            }
        
            $this->entityManager->commit();
            return $parcnegocioContato;
        }
        catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function deleteContato($row){
        $nrorg = $this->environment->getNrorgTrab();
        $operador = $this->environment->getCdOperador();
        $currentTime = $this->environment->getCurrentTime();

        $relacionaparcRepository = $this->entityManager->getRepository(Relacionaparc::class);
        $comunicaparcRepository  = $this->entityManager->getRepository(Comunicaparc::class);
        $parcnegocioRepository   = $this->entityManager->getRepository(Parcnegocio::class);
        
        $this->entityManager->beginTransaction();
        if($row['TIPODEPARCEIRO'] === 'INTERLOCUTOR' && $row['PARENTTYPE'] === 'PROSPECT'){
            $this->parcNegocioFactory->deleteInteprosp($row['NRRELACIONAPARC'], $row['NRORG']);
        }
        if (!empty($row["NRPARCNEGRELAC"])) {
            $parcnegocio = $parcnegocioRepository->findOneBy(array(
               "nrparcnegocio" => $row["NRPARCNEGRELAC"],
               "nrorg" => $nrorg
            ));
            
            $comunica = $comunicaparcRepository->findBy(array(
               "nrparcnegocio" => $row["NRPARCNEGRELAC"],
               "nrorg" => $nrorg
            ));
            if($comunica !== null){
                foreach ($comunica as $comunicacao) {
                    $this->entityManager->remove($comunicacao);
                    $this->entityManager->flush();
                }
            }
            
            $relaciona = $relacionaparcRepository->findBy(array(
               "nrrelacionaparc" => $row["NRRELACIONAPARC"],
               "nrorg" => $nrorg
            ));
            if($relaciona !== null){
                foreach ($relaciona as $relacao) {
                    $this->entityManager->remove($relacao);
                    $this->entityManager->flush();
                }
            }
            
            if($parcnegocio !== null){
                $this->entityManager->remove($parcnegocio);
                $this->entityManager->flush();    
            }
        }    
        $this->entityManager->commit();
    }

    /*
     *
     *  Funções de validação de dados
     *
     */
    private function isNewItem(Parceria $parceria) {
        if (($parceria->getNrparceria() == null) &&
           ($parceria->getNrparcnegocio() != null) &&
           ($parceria->getCdtipoparceiro() != null)) {
            return true;
        }
        else {
            return false;
        }
    }

    // Método que verifica se o tipo de parceiro principal enviado na requisição
    // é correto, de acordo com a ordem de precedência dos tipos.
    private function tipoPrincipalIsValid($tipos, $principalEnviado) {
        $tipoPrincipal = null;
        $i = 0;
        while ($tipoPrincipal === null && $i < count($this->defaultTipos)) {
            if (in_array($this->defaultTipos[$i], $tipos)) {
                $tipoPrincipal = $this->defaultTipos[$i];
            }
            $i++;
        }
        if ($tipoPrincipal === null || $tipoPrincipal != $principalEnviado) {
            return false;
        }
        return true;
    }

    private function validateParceiro($row) {
        if (empty($row["CDTIPOINSCRICAO"])) {
            throw new \Exception("Tipo inscrição não informado.");
        }
        if (empty($row["NRINSCRICAOPARC"])) {
            throw new \Exception("Nº inscrição não informado.");
        }
        if (empty($row["NMPRINCIPALPARC"])) {
            throw new \Exception("Razão Social/Nome Completo não informado.");
        }
        if (empty($row["NMSECUNDARIPARC"])) {
            throw new \Exception("Nome Fantasia/Nome Reduzido não informado.");
        }
        if (empty($row["IDINSTITUICAO"]) || ($row["IDINSTITUICAO"] != 'S' && $row["IDINSTITUICAO"] != 'N')) {
            throw new \Exception("ID de Instituição não informado.");
        }
        if (empty($row["IDPESSOAFISICA"]) || ($row["IDPESSOAFISICA"] != 'S' && $row["IDPESSOAFISICA"] != 'N')) {
            throw new \Exception("ID de Pessoa Física não informado.");
        }
        if ($row["IDINSTITUICAO"] == $row["IDPESSOAFISICA"]) {
            throw new \Exception("ID de Pessoa Física inválido.");
        }
        if (empty($row["TIPODEPARCEIRO"])) {
            throw new \Exception("Tipo de Parceiro não informado.");
        }
        if (empty($row["CDTIPOPARCPRINCIPAL"])) {
            throw new \Exception("Tipo Principal não informado.");
        }
        if (in_array("TRANSPORTADORA", $row["TIPODEPARCEIRO"])) {
            if (empty($row["NRINSESTTRAN"])) {
                throw new \Exception("Inscrição Estadual não informada.");
            }
            if (empty($row["ENDERECOTRAN"])) {
                throw new \Exception("Endereço não informado.");
            }
            if (empty($row["NMBAIRROTRAN"])) {
                throw new \Exception("Bairro não informado.");
            }
            if (empty($row["NRCEPTRAN"])) {
                throw new \Exception("CEP não informado.");
            }
            if (empty($row["CDPAIS"])) {
                throw new \Exception("País não informado.");
            }
            if (empty($row["SGESTADO"])) {
                throw new \Exception("Estado não informado.");
            }
            if (empty($row["CDMUNICIP"])) {
                throw new \Exception("Município não informado.");
            }
        }
        if(!array_key_exists('contato', $row)){
            if ($row["__is_new"] || $row["inscricaoWasEdited"]) {
                $nrorg = $this->environment->getNrorgTrab();
                $nrinscricaoparc = $row["NRINSCRICAOPARC"];
                $cdtipoinscricao = $row["CDTIPOINSCRICAO"];
                $parcnegocio = $this->entityManager->getRepository(Parcnegocio::class)->findOneBy(array(
                    "nrinscricaoparc" => $nrinscricaoparc,
                    "cdtipoinscricao" => $cdtipoinscricao,
                    "nrorg" => $nrorg
                ));
                if ($parcnegocio !== null) {
                    throw new \Exception("A inscrição inserida já pertence ao parceiro " . $parcnegocio->getNmprincipalparc());
                }
            }
        }
    }
    
    public function atualizarCaminhoLogo($nrparcnegocio, $nrorg, $dsdirlogomarca = null) {
        $params = array(
            "DSDIRLOGOMARCA" => $dsdirlogomarca,
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRORG"          => $nrorg
        );
        
        $this->parcNegocioFactory->atualizarCaminhoLogo($params);
    }
    
     /**
     * A função "completaPercentual" verifica qual o percentual faltante para inserção na página "índices Financeiros Padrões"
     */
    
    public function completaPercentual($nrparcnegocio, $nrorg){
        
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG" => $nrorg
        );
        
        $valida = $this->parcNegocioFactory->validaPercentual($params);
        
        if (!empty($valida[0])){
            
           $result = 100 - $valida[0]["TOTPERCENT"];
            
        }
        else{
           
           $result =  100; 
        }
        
        return $result;
    }
    
     
     /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual($nrparcnegocio, $nrindicespadrparc, $nrorg){
        $params = array(
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "NRORG"             => $nrorg,
            "NRINDICESPADRPARC" => $nrindicespadrparc
        );
        
        $valida = $this->parcNegocioFactory->validaPercentual($params);
        return (float)$valida[0]["TOTPERCENT"];
    }
    
    /**
     * A função "saveParamFatura" verifica se existe o registro para o parceiro de negócio enviado. Sendo que:
     *   - Caso não exista, cria o registro. 
     *   - Caso exista, altera o registro.
     */
    
    public function saveParamFatura($row, $nrorg){
        
        $params = array(
            "NRPARCNEGOCIO" => $row["NRPARCNEGOCIO"],
            "NRORG" => $nrorg
        );
        
        $valida = $this->parcNegocioFactory->validaRegistoParamFat($params);
        
        if ($valida > 0){
            
            $params = array(
                "CDFILIALFATURA" => $row["CDFILIALFATURA"],
                "CDSERIENF"      => $row["CDSERIENF"],
                "CDEMPRESA"      => $row["CDEMPRESA"],
                "CDINSCESTA"     => $row["CDINSCESTA"],
                "CDINSCMUNI"     => $row["CDINSCMUNI"],
                "NRORG"          => $nrorg,
                "NRPARCNEGOCIO"  => $row["NRPARCNEGOCIO"]
            );
            
            $this->parcNegocioFactory->updateParParamFatura($params);
        }
        else{
           
           $nrparamfatura = $this->newCode->getCodeByTableName('PAR_PARAMFATURA', $nrorg, 1, 10, true);
           
           $params = array(
                "NRPARAMFATURA"  => $nrparamfatura,
                "NRPARCNEGOCIO"  => $row["NRPARCNEGOCIO"],
                "CDFILIALFATURA" => $row["CDFILIALFATURA"],
                "CDSERIENF"      => $row["CDSERIENF"],
                "CDEMPRESA"      => $row["CDEMPRESA"],
                "CDINSCESTA"     => $row["CDINSCESTA"],
                "CDINSCMUNI"     => $row["CDINSCMUNI"],
                "NRORG"          => $nrorg
            );
            
            $this->parcNegocioFactory->insertParParamFatura($params);
        }
    }
    
    /* recebe os dados de parâmetro de Faturamento */
    public function recebeParamFatura($nrparcnegocio, $nrorg){
        
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG" => $nrorg
        );
        
        $dados = $this->parcNegocioFactory->recebeParamFatura($params);
        
        if (!empty($dados)){
            
            return $dados;
        }
        else {
            return null;
        }
        
    }
}
